@@ -22,7 +22,8 @@
 ##' derivatives.
 ##'
 ##' It is an implementation of the closed-form C-spline basis derived from the
-##' recursion formula of I-splines and M-splines.
+##' recursion formula of I-splines and M-splines.  The function \code{csp()} is
+##' an alias of to encourage the use in a model formula.
 ##'
 ##' @inheritParams bSpline
 ##'
@@ -42,9 +43,10 @@
 ##'     height at right boundary knot. The corresponding I-spline and M-spline
 ##'     produced by \code{deriv} methods will be scaled to the same extent.
 ##'
-##' @inherit bSpline return
+##' @inherit iSpline return
 ##'
 ##' @references
+##'
 ##' Meyer, M. C. (2008). Inference using shape-restricted regression splines.
 ##' \emph{The Annals of Applied Statistics}, 2(3), 1013--1033.
 ##'
@@ -57,7 +59,9 @@
 ##' @export
 cSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
                     intercept = TRUE, Boundary.knots = NULL,
-                    derivs = 0L, scale = TRUE, ...)
+                    derivs = 0L, scale = TRUE,
+                    warn.outside = getOption("splines2.warn.outside", TRUE),
+                    ...)
 {
     ## check inputs
     if ((derivs <- as.integer(derivs)) < 0) {
@@ -93,8 +97,8 @@ cSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
                    degree = degree,
                    internal_knots = knots,
                    boundary_knots = Boundary.knots,
-                   derivs = derivs,
-                   complete_basis = intercept
+                   complete_basis = intercept,
+                   derivs = derivs
                )
            } else {
                if (derivs == 0) {
@@ -104,9 +108,9 @@ cSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
                        degree = degree,
                        internal_knots = knots,
                        boundary_knots = Boundary.knots,
+                       complete_basis = intercept,
                        derivs = 0,
-                       integral = TRUE,
-                       complete_basis = intercept
+                       integral = TRUE
                    )
                } else {
                    rcpp_iSpline(
@@ -115,15 +119,15 @@ cSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
                        degree = degree,
                        internal_knots = knots,
                        boundary_knots = Boundary.knots,
+                       complete_basis = intercept,
                        derivs = derivs - 1,
-                       integral = FALSE,
-                       complete_basis = intercept
+                       integral = FALSE
                    )
                }
            }
     ## throw warning if any x is outside of the boundary
     b_knots <- attr(out, "Boundary.knots")
-    if (any((xx < b_knots[1L]) | (xx > b_knots[2L]))) {
+    if (warn.outside && any((xx < b_knots[1L]) | (xx > b_knots[2L]))) {
         warning(wrapMessages(
             "Some 'x' values beyond boundary knots",
             "may cause ill-conditioned basis functions."
@@ -148,12 +152,16 @@ cSpline <- function(x, df = NULL, knots = NULL, degree = 3L,
     if (scale || derivs == 0) {
         ## add "scale" to attributes for predict(), etc.
         attr(out, "scale") <- scale
-        class(out) <- c("matrix", "cSpline", "splines2")
+        class(out) <- c("CSpline", "splines2", "matrix")
     } else if (derivs == 1) {
-        class(out) <- c("matrix", "iSpline", "splines2")
+        class(out) <- c("ISpline", "splines2", "matrix")
     } else {
-        class(out) <- c("matrix", "mSpline", "splines2")
+        class(out) <- c("MSpline", "splines2", "matrix")
     }
     ## return
     out
 }
+
+##' @rdname cSpline
+##' @export
+csp <- cSpline
