@@ -1,6 +1,7 @@
 ## get implementations of v0.2.8 for reference
 v2 <- new.env()
 source("../v0.2.8.R", v2)
+source("utils.R")
 
 ## helper functions
 isNumMatrix <- v2$isNumMatrix
@@ -43,6 +44,14 @@ expect_true(isNumMatrix(nsMat1, length(x), length(knots) + 2L))
 expect_true(isNumMatrix(nsMat2, length(x), length(knots) + 2L))
 expect_true(isNumMatrix(nsMat3, length(x), length(knots) + 2L))
 
+## check the returned H matrix in attribute
+h_mat <- attr(nsMat0, "H")
+bsMat <- bSpline(x, knots = knots, intercept = TRUE)
+expect_eqt(nsMat0, bsMat %*% h_mat)
+expect_eqt(nsMat1, ibs(x, knots = knots, intercept = TRUE) %*% h_mat)
+expect_eqt(nsMat2, deriv(bsMat) %*% h_mat)
+expect_eqt(nsMat3, deriv(bsMat, 2) %*% h_mat)
+
 ## specify df directly instead of knots
 for (j in seq.int(2, 10)) {
     expect_true(isNumMatrix(
